@@ -12,6 +12,7 @@ __all__ = ["Event", "Lock", "RLock", "Condition", "Semaphore",
            "BoundedSemaphore", "Timer", "Local", "Thread", "Queue",
            "LifoQueue", "PriorityQueue", "Counter"]
 
+
 def _debugger(cls):
     import types
     for name in dir(cls):
@@ -20,16 +21,17 @@ def _debugger(cls):
             def extrascope(attr):
                 @functools.wraps(attr)
                 def wrapper(*args, **kwargs):
-                    print "%s.%s %s %s" % (cls.__name__, attr.__name__,
-                            repr(args[1:]), repr(kwargs))
+                    print "%s(%x).%s %s %s" % (cls.__name__, id(args[0]),
+                            attr.__name__, repr(args[1:]), repr(kwargs))
                     rc = attr(*args, **kwargs)
-                    print "%s.%s --> %s" % (cls.__name__, attr.__name__,
-                            repr(rc))
+                    print "%s(%x).%s --> %s" % (cls.__name__, id(args[0]),
+                            attr.__name__, repr(rc))
                     return rc
                 return wrapper
             setattr(cls, name, extrascope(attr))
     return cls
 
+
 #@_debugger
 class Event(object):
     """an event for which greenlets can wait
@@ -89,7 +91,7 @@ class Event(object):
         if self._is_set:
             return False
 
-        current = compat.getcurrent() # the waiting greenlet
+        current = compat.getcurrent()  # the waiting greenlet
 
         waketime = None if timeout is None else time.time() + timeout
         if timeout is not None:
@@ -100,11 +102,14 @@ class Event(object):
 
         if timeout is not None:
             if not scheduler._remove_timer(waketime, current):
-                self._waiters.remove(current)
+                scheduler.state.awoken_from_events.discard(current)
+                if current in self._waiters:
+                    self._waiters.remove(current)
                 return True
 
         return False
 
+
 #@_debugger
 class Lock(object):
     """an object that can only be 'owned' by one greenlet at a time
@@ -190,6 +195,7 @@ class Lock(object):
     def __exit__(self, type, value, traceback):
         return self.release()
 
+
 class RLock(Lock):
     """a lock which may be acquired more than once by the same greenlet
 
@@ -260,6 +266,7 @@ class RLock(Lock):
                 self._locked = False
                 self._owner = None
 
+
 class Condition(object):
     """a synchronization object capable of waking all or one of its waiters
 
@@ -348,6 +355,7 @@ class Condition(object):
         self._waiters.clear()
     notifyAll = notify_all
 
+
 class Semaphore(object):
     """a synchronization object with a counter that blocks when it reaches 0
 
@@ -402,6 +410,7 @@ class Semaphore(object):
     def __exit__(self, type, value, traceback):
         return self.release()
 
+
 class BoundedSemaphore(Semaphore):
     """a semaphore with an upper limit to the counter
 
@@ -430,6 +439,7 @@ class BoundedSemaphore(Semaphore):
         return super(BoundedSemaphore, self).release()
     release.__doc__ = Semaphore.release.__doc__
 
+
 class Local(object):
     """an object that holds greenlet-local data
 
@@ -451,7 +461,7 @@ class Local(object):
             current = self._main_standin
         local = self._local_data.setdefault(current, {})
         if name not in local:
-            raise AttributeError, "Local object has no attribute %s" % name
+            raise AttributeError("Local object has no attribute %s" % name)
         return local[name]
 
     def __setattr__(self, name, value):
@@ -460,6 +470,7 @@ class Local(object):
             current = self._main_standin
         self._local_data.setdefault(current, {})[name] = value
 
+
 class Thread(object):
     """a standin class for threads, but powered by greenlets
 
@@ -492,7 +503,7 @@ class Thread(object):
             args=(),
             kwargs=None,
             verbose=None):
-        assert group is None, "group argument must be None for now" #[sic]
+        assert group is None, "group argument must be None for now"  # [sic]
 
         self._target = target
         self.name = str(name or self._newname())
@@ -529,6 +540,7 @@ class Thread(object):
         """
         if self._started:
             raise RuntimeError("thread already started")
+
         def run():
             try:
                 self.run(*self._args, **self._kwargs)
@@ -537,6 +549,7 @@ class Thread(object):
                 pass
             finally:
                 self._deactivate()
+
         self._glet = scheduler.greenlet(run)
         self._ident = id(self._glet)
         scheduler.schedule(self._glet)
